'use strict';

const http = require('http');
const queryString = require('query-string');

const Request = http.IncomingMessage.prototype;
const Response = http.ServerResponse.prototype;

Object.defineProperty(Request, 'params', { get: function() {
  return queryString(this.url.split('?')[1]);
}});

Object.defineProperty(Request, 'body', { get: async function() {
  if(this._body) 
    return this._body;
  
  this._body = await (new Promise((resolve, reject) => {
    this.on('data', c => resolve(c.toString()))
    .on('end', () => resolve('{}'))
  }));
  
  return this._body;
}});

class Speedlane {
  constructor(port) {
    this._routes = {};
    this._middleware = [];
    
    http.METHODS.forEach(m => this._routes[m] = {});
    
    http.createServer(async(req, res) => {
      const cbs = Object.create(this._middleware);
      const route = this._routes[req.method][req.url.split('?')[0]];
      if(route)
        cbs.unshift(route);
      
      this._run(cbs, req, res);
    }).listen(port);
  }
  
  on(method, path, cb) {
    this._routes[method][path] = cb;
    return this;
  }
  
  use(cb) {
    this._middleware.unshift(cb);
    return this;
  }
  
  _run(cbs, req, res) {
    (async function next() {
      if(cbs.length) 
        await cbs.pop()(req, res, next);
    })();
  }
}

module.exports = { Speedlane, Request, Response };